/*
	This simple extension of the java.awt.Frame class
	contains all the elements necessary to act as the
	main window of an application.
 */


/** 
 * This is a very simple example of a 'simulated' shower with a hot and a cold
 * valve to control the flow and temperature in a shower. The simulation
 * is extremely crude and does not take into account delays in mixing
 * and travel of the hot and cold water or delays required to move the
 * valves. So we have perfect mixing and perfect and immediate results
 * to changes in the valve positions.The intent is to show a fuzzy system that
 * with a few rules that do not 'cover' the space well, can still do a reasonable
 * job of controlling the shower. Ideally we would have more terms for the 
 * shower temperature and pressure, therefore more rules, and we would
 * monitor the behaviour of the system so that it could tune the rules
 * based on performance of the system. For example, if the pressures of the
 * hot and cold rise, the adjustments will affect bigger system responses
 * so the rules should be tempered to account for the new sensitivity.
 *
 */
package examples.fuzzyshowerjess;

import java.awt.*;

import symantec.itools.awt.HorizontalSlider;
import symantec.itools.awt.Label3D;
import symantec.itools.awt.shape.Line;
import symantec.itools.awt.shape.VerticalLine;
import symantec.itools.awt.ImagePanel;
import nrc.fuzzy.*;

import symantec.itools.awt.util.spinner.NumericSpinner;
public class ShowerFrameJess extends Frame
{
    InferenceCycleFrameJess inferenceCycleResults;
    ShowerSimulateJess showerSimulator;
    
    String applicationDirectory = ""; // directory where shower.gif file is
    
	public ShowerFrameJess(String title, String appDir)
	{
		applicationDirectory = appDir;
		
		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		
		//{{INIT_CONTROLS
		setLayout(null);
		setVisible(false);
		setSize(681,538);
		setForeground(new Color(0));
		setBackground(new Color(12632256));
		hotPressureSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			hotPressureSlider.setMaxValue(100);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotPressureSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotPressureSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotPressureSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		hotPressureSlider.setBounds(48,96,130,48);
		add(hotPressureSlider);
		hotValveSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			hotValveSlider.setMaxValue(99);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotValveSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotValveSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotValveSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		hotValveSlider.setBounds(48,216,130,48);
		add(hotValveSlider);
		hotTempSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			hotTempSlider.setMaxValue(100);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotTempSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotTempSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotTempSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		hotTempSlider.setBounds(48,324,130,48);
		add(hotTempSlider);
		coldPressureSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			coldPressureSlider.setMaxValue(99);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldPressureSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldPressureSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldPressureSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		coldPressureSlider.setBounds(540,96,130,48);
		add(coldPressureSlider);
		coldValveSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			coldValveSlider.setMaxValue(99);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldValveSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldValveSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldValveSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		coldValveSlider.setBounds(540,216,130,48);
		add(coldValveSlider);
		coldTempSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			coldTempSlider.setMaxValue(99);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldTempSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldTempSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldTempSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		coldTempSlider.setBounds(540,324,130,48);
		add(coldTempSlider);
		showerTempSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			showerTempSlider.setMaxValue(72);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerTempSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerTempSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerTempSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		showerTempSlider.setBounds(216,12,108,48);
		add(showerTempSlider);
		showerTempSlider.setEnabled(false);
		showerFlowSlider = new symantec.itools.awt.HorizontalSlider();
		try {
			showerFlowSlider.setMaxValue(24);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerFlowSlider.setMinValue(0);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerFlowSlider.setShowBorder(false);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerFlowSlider.setTickStyle(symantec.itools.awt.HorizontalSlider.TICK_NONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		showerFlowSlider.setBounds(396,12,108,48);
		add(showerFlowSlider);
		showerFlowSlider.setEnabled(false);
		hotPressureLabel3D = new symantec.itools.awt.Label3D();
		try {
			hotPressureLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotPressureLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		hotPressureLabel3D.setBounds(12,108,32,20);
		add(hotPressureLabel3D);
		hotValveLabel3D = new symantec.itools.awt.Label3D();
		try {
			hotValveLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotValveLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		hotValveLabel3D.setBounds(12,228,32,20);
		add(hotValveLabel3D);
		hotTempLabel3D = new symantec.itools.awt.Label3D();
		try {
			hotTempLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			hotTempLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		hotTempLabel3D.setBounds(12,336,32,20);
		add(hotTempLabel3D);
		coldPressureLabel3D = new symantec.itools.awt.Label3D();
		try {
			coldPressureLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldPressureLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		coldPressureLabel3D.setBounds(504,108,32,20);
		add(coldPressureLabel3D);
		coldValveLabel3D = new symantec.itools.awt.Label3D();
		try {
			coldValveLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldValveLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		coldValveLabel3D.setBounds(504,228,32,20);
		add(coldValveLabel3D);
		coldTempLabel3D = new symantec.itools.awt.Label3D();
		try {
			coldTempLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			coldTempLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		coldTempLabel3D.setBounds(504,336,32,20);
		add(coldTempLabel3D);
		showerTempLabel3D = new symantec.itools.awt.Label3D();
		try {
			showerTempLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerTempLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		showerTempLabel3D.setBounds(180,24,32,20);
		add(showerTempLabel3D);
		showerFlowLabel3D = new symantec.itools.awt.Label3D();
		try {
			showerFlowLabel3D.setBevelStyle(symantec.itools.awt.Label3D.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			showerFlowLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		showerFlowLabel3D.setBounds(360,24,32,20);
		add(showerFlowLabel3D);
		label1 = new java.awt.Label("Hot Pressure (kPa)",Label.CENTER);
		label1.setBounds(60,144,108,22);
		label1.setForeground(new Color(16711680));
		add(label1);
		label2 = new java.awt.Label("Hot Valve (%)",Label.CENTER);
		label2.setBounds(60,264,108,22);
		label2.setForeground(new Color(16711680));
		add(label2);
		label3 = new java.awt.Label("Hot Temperature (C)",Label.CENTER);
		label3.setBounds(48,372,120,22);
		label3.setForeground(new Color(16711680));
		add(label3);
		label4 = new java.awt.Label("Cold Pressure (kPa)",Label.CENTER);
		label4.setBounds(540,144,120,22);
		label4.setForeground(new Color(255));
		add(label4);
		label5 = new java.awt.Label("Cold Temperature (C)",Label.CENTER);
		label5.setBounds(540,372,120,22);
		label5.setForeground(new Color(255));
		add(label5);
		label6 = new java.awt.Label("Cold Valve (%)",Label.CENTER);
		label6.setBounds(552,264,108,22);
		label6.setForeground(new Color(255));
		add(label6);
		label7 = new java.awt.Label(" Shower Temperature (C)",Label.RIGHT);
		label7.setBounds(156,60,168,22);
		label7.setFont(new Font("Dialog", Font.BOLD, 12));
		add(label7);
		label8 = new java.awt.Label("Shower Flow (litres/min)",Label.RIGHT);
		label8.setBounds(360,60,144,22);
		label8.setFont(new Font("Dialog", Font.BOLD, 12));
		add(label8);
		noticeLabel3D = new symantec.itools.awt.Label3D();
		try {
			noticeLabel3D.setText("Manual Mode: Set Temperatures and Pressures of Hot and Cold feeds and select auto for fuzzy control");
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			noticeLabel3D.setBorderIndent(symantec.itools.awt.Label3D.INDENT_ONE);
		}
		catch(java.beans.PropertyVetoException e) { }
		try {
			noticeLabel3D.setAlignStyle(symantec.itools.awt.Label3D.ALIGN_LEFT);
		}
		catch(java.beans.PropertyVetoException e) { }
		noticeLabel3D.setBounds(48,504,600,25);
		noticeLabel3D.setFont(new Font("Dialog", Font.BOLD, 12));
		add(noticeLabel3D);
		autoCheckbox = new java.awt.Checkbox("auto Fuzzy Control");
		autoCheckbox.setBounds(312,456,132,24);
		add(autoCheckbox);
		imagePanel1 = new symantec.itools.awt.ImagePanel();
		try {
			java.net.URL url;
			if (applicationDirectory.equals(new String("")))
			    url = symantec.itools.net.RelativeURL.getURL("shower.gif");
			else
			    url = new java.net.URL("file:/" + applicationDirectory + "shower.gif");
			imagePanel1.setImageURL(url);
		}
		catch (java.net.MalformedURLException error) 
		{ System.out.println("*** ERROR *** " + error); }
		catch(java.beans.PropertyVetoException e) { }
		try {
			imagePanel1.setStyle(symantec.itools.awt.ImagePanel.IMAGE_SCALED_TO_FIT);
		}
		catch(java.beans.PropertyVetoException e) { }
		imagePanel1.setLayout(null);
		imagePanel1.setBounds(192,108,288,300);
		add(imagePanel1);
		label9 = new java.awt.Label("Millisecs between inference cycles");
		label9.setBounds(24,456,192,22);
		add(label9);
		msSleepTextField = new java.awt.TextField();
		msSleepTextField.setText("0");
		msSleepTextField.setBounds(216,456,59,29);
		add(msSleepTextField);
		showFiringsCheckbox = new java.awt.Checkbox("show rule firings");
		showFiringsCheckbox.setBounds(492,456,121,22);
		add(showFiringsCheckbox);
		setTitle("Fuzzy Shower Demo");
		//}}
		
		//{{INIT_MENUS
		mainMenuBar = new java.awt.MenuBar();
		menu1 = new java.awt.Menu("File");
		miExit = new java.awt.MenuItem("Exit");
		menu1.add(miExit);
		mainMenuBar.add(menu1);
		menu3 = new java.awt.Menu("Help");
		mainMenuBar.setHelpMenu(menu3);
		miAbout = new java.awt.MenuItem("About..");
		menu3.add(miAbout);
		mainMenuBar.add(menu3);
		setMenuBar(mainMenuBar);
		//$$ mainMenuBar.move(0,420);
		//}}
		
		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		miAbout.addActionListener(lSymAction);
		miExit.addActionListener(lSymAction);
		SymPropertyChange lSymPropertyChange = new SymPropertyChange();
		hotPressureSlider.addPropertyChangeListener(lSymPropertyChange);
		hotPressureSlider.addActionListener(lSymAction);
		hotValveSlider.addPropertyChangeListener(lSymPropertyChange);
		hotValveSlider.addActionListener(lSymAction);
		hotTempSlider.addActionListener(lSymAction);
		hotTempSlider.addPropertyChangeListener(lSymPropertyChange);
		coldPressureSlider.addActionListener(lSymAction);
		coldPressureSlider.addPropertyChangeListener(lSymPropertyChange);
		coldValveSlider.addActionListener(lSymAction);
		coldValveSlider.addPropertyChangeListener(lSymPropertyChange);
		coldTempSlider.addPropertyChangeListener(lSymPropertyChange);
		coldTempSlider.addActionListener(lSymAction);
		showerTempSlider.addPropertyChangeListener(lSymPropertyChange);
		showerTempSlider.addActionListener(lSymAction);
		showerFlowSlider.addPropertyChangeListener(lSymPropertyChange);
		showerFlowSlider.addActionListener(lSymAction);
		SymItem lSymItem = new SymItem();
		autoCheckbox.addItemListener(lSymItem);
		showFiringsCheckbox.addItemListener(lSymItem);
		showerTempLabel3D.addPropertyChangeListener(lSymPropertyChange);
		showerFlowLabel3D.addPropertyChangeListener(lSymPropertyChange);
		//}}
		
		setTitle(title);
	}
	
	public ShowerFrameJess(String title)
	{
		this(title, "");
	}
	
    /**
     * Shows or hides the component depending on the boolean flag b.
     * @param b  if true, show the component; otherwise, hide the component.
     * @see java.awt.Component#isVisible
     */
    public void setVisible(boolean b)
	{
		if(b)
		{
			setLocation(50, 50);
		}	
		super.setVisible(b);
	}
	
	static public void main(String args[])
	{   // args is either empty or it has a string identifiying
		// where the directory is for the shower.gif file and
		// the FuzzyShowerJess.clp file.
		String appDir = "";
		if (args.length > 0)
		{	appDir = args[0];
		}
	    ShowerFrameJess showerFrame = new ShowerFrameJess("", appDir);
	    showerFrame.setVisible(true);
	    
	    showerFrame.inferenceCycleResults = new InferenceCycleFrameJess();
	    showerFrame.showerSimulator = new ShowerSimulateJess(showerFrame, null);
	    showerFrame.showerSimulator.initSliders();
		showerFrame.repaint();
	}
	
	public void setNoticeLabel(String text)
	{
	    try
	    {   noticeLabel3D.setText(text);
	    }
	    catch (Exception e)
	    {   System.err.println("Error setting notice label.\n" +e);
	    }
	}
	
			
	public void addNotify()
	{
		// Record the size of the window prior to calling parents addNotify.
		Dimension d = getSize();
		
		super.addNotify();
	
		if (fComponentsAdjusted)
			return;
	
		// Adjust components according to the insets
		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}
	
	// Used for addNotify check.
	boolean fComponentsAdjusted = false;
	
	//{{DECLARE_CONTROLS
	symantec.itools.awt.HorizontalSlider hotPressureSlider;
	symantec.itools.awt.HorizontalSlider hotValveSlider;
	symantec.itools.awt.HorizontalSlider hotTempSlider;
	symantec.itools.awt.HorizontalSlider coldPressureSlider;
	symantec.itools.awt.HorizontalSlider coldValveSlider;
	symantec.itools.awt.HorizontalSlider coldTempSlider;
	symantec.itools.awt.HorizontalSlider showerTempSlider;
	symantec.itools.awt.HorizontalSlider showerFlowSlider;
	symantec.itools.awt.Label3D hotPressureLabel3D;
	symantec.itools.awt.Label3D hotValveLabel3D;
	symantec.itools.awt.Label3D hotTempLabel3D;
	symantec.itools.awt.Label3D coldPressureLabel3D;
	symantec.itools.awt.Label3D coldValveLabel3D;
	symantec.itools.awt.Label3D coldTempLabel3D;
	symantec.itools.awt.Label3D showerTempLabel3D;
	symantec.itools.awt.Label3D showerFlowLabel3D;
	java.awt.Label label1;
	java.awt.Label label2;
	java.awt.Label label3;
	java.awt.Label label4;
	java.awt.Label label5;
	java.awt.Label label6;
	java.awt.Label label7;
	java.awt.Label label8;
	symantec.itools.awt.Label3D noticeLabel3D;
	java.awt.Checkbox autoCheckbox;
	symantec.itools.awt.ImagePanel imagePanel1;
	java.awt.Label label9;
	java.awt.TextField msSleepTextField;
	java.awt.Checkbox showFiringsCheckbox;
	//}}
	
	//{{DECLARE_MENUS
	java.awt.MenuBar mainMenuBar;
	java.awt.Menu menu1;
	java.awt.MenuItem miExit;
	java.awt.Menu menu3;
	java.awt.MenuItem miAbout;
	//}}
	
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == ShowerFrameJess.this)
				Frame1_WindowClosing(event);
		}
	}
	
	void Frame1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);	// hide the Frame
		dispose();			// free the system resources
		System.exit(0);		// close the application
	}
	
	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
            if (object == miAbout)
				miAbout_Action(event);
			else if (object == miExit)
				miExit_Action(event);
			else if (object == hotPressureSlider)
				hotPressureSlider_actionPerformed(event);
			else if (object == hotValveSlider)
				hotValveSlider_actionPerformed(event);
			else if (object == hotTempSlider)
				hotTempSlider_actionPerformed(event);
			else if (object == coldPressureSlider)
				coldPressureSlider_actionPerformed(event);
			else if (object == coldValveSlider)
				coldValveSlider_actionPerformed(event);
			else if (object == coldTempSlider)
				coldTempSlider_actionPerformed(event);
			else if (object == showerTempSlider)
				showerTempSlider_actionPerformed(event);
			else if (object == showerFlowSlider)
				showerFlowSlider_actionPerformed(event);
		}
	}
	
	void miAbout_Action(java.awt.event.ActionEvent event)
	{
		//{{CONNECTION
		// Action from About Create and show as modal
		(new AboutDialogJess(this, true)).setVisible(true);
		//}}
	}
	
	void miExit_Action(java.awt.event.ActionEvent event)
	{
		//{{CONNECTION
		// Action from Exit Create and show as modal
		(new QuitDialogJess(this, true)).setVisible(true);
		//}}
	}
	

	class SymPropertyChange implements java.beans.PropertyChangeListener
	{
		public void propertyChange(java.beans.PropertyChangeEvent event)
		{
			Object object = event.getSource();
			if (object == hotPressureSlider)
				hotPressureSlider_propertyChange(event);
			else if (object == hotValveSlider)
				hotValveSlider_propertyChange(event);
			else if (object == hotTempSlider)
				hotTempSlider_propertyChange(event);
			else if (object == coldPressureSlider)
				coldPressureSlider_propertyChange(event);
			else if (object == coldValveSlider)
				coldValveSlider_propertyChange(event);
			else if (object == coldTempSlider)
				coldTempSlider_propertyChange(event);
			else if (object == showerTempSlider)
				showerTempSlider_propertyChange(event);
			else if (object == showerFlowSlider)
				showerFlowSlider_propertyChange(event);
		}
	}

	void hotPressureSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ hotPressureLabel3D.setText(String.valueOf(hotPressureSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void hotPressureSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ 
    		// keep pressure between 35 and 90
    		double hotPressure = (double)hotPressureSlider.getValue();
    		if (hotPressure < 35.0)
    		{ hotPressure = 35.0;
    		  hotPressureSlider.setValue((int)Math.round(hotPressure));
    		}
    		if (hotPressure > 90.0)
    		{ hotPressure = 90.0;
    		  hotPressureSlider.setValue((int)Math.round(hotPressure));
    		}
    		showerSimulator.setHotPressure(hotPressure);
		    hotPressureLabel3D.setText(String.valueOf(hotPressureSlider.getValue()));
		}
		catch (Exception e)
		{ }		
		if (autoCheckbox.getState() == false)
		    showerSimulator.setOutFlowAndOutTemp();
		else
		    showerSimulator.simulateShower();
		repaint();
	}

	void hotValveSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ 
		    hotValveLabel3D.setText(String.valueOf(hotValveSlider.getValue()));
		}
		catch (Exception e)
		{ }
	}


	void hotValveSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ 
		    if (autoCheckbox.getState() == false)
		    {   // manual mode ... set outFlow and outTemp
		        double hotValve = hotValveSlider.getValue()/100.0;
    		    showerSimulator.setHotValve(hotValve);
    		    showerSimulator.setOutFlowAndOutTemp();
    		}
		    else
		        // auto mode ... can't set the valve position manually
		        hotValveSlider.setValue((int)Math.round(showerSimulator.getHotValve()*100.0));
		    
		    hotValveLabel3D.setText(String.valueOf(hotValveSlider.getValue()));
		}
		catch (Exception e)
		{ }
		repaint();
	}

	void hotTempSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ 		
    		// keep temp between 5 and 100
    		double hotTemp = (double)hotTempSlider.getValue();
    		if (hotTemp < 5.0)
    		{ hotTemp = 5.0;
    		  hotTempSlider.setValue((int)Math.round(hotTemp));
    		}
    		if (hotTemp > 100.0)
    		{ hotTemp = 100.0;
    		  hotTempSlider.setValue((int)Math.round(hotTemp));
    		}
    		showerSimulator.setHotTemp(hotTemp);
		    hotTempLabel3D.setText(String.valueOf(hotTempSlider.getValue()));
		}
		catch (Exception e)
		{ }
		
		if (autoCheckbox.getState() == false)
		    showerSimulator.setOutFlowAndOutTemp();
		else
		    showerSimulator.simulateShower();
		repaint();
	}

	void hotTempSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ hotTempLabel3D.setText(String.valueOf(hotTempSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void coldPressureSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ 
    		// keep pressure between 30 and 90
    		double coldPressure = (double)coldPressureSlider.getValue();
    		if (coldPressure < 35.0)
    		{ coldPressure = 35.0;
    		  coldPressureSlider.setValue((int)Math.round(coldPressure));
    		}
    		if (coldPressure > 90.0)
    		{ coldPressure = 90.0;
    		  coldPressureSlider.setValue((int)Math.round(coldPressure));
    		}
    		showerSimulator.setColdPressure(coldPressure);
		    coldPressureLabel3D.setText(String.valueOf(coldPressureSlider.getValue()));
		}
		catch (Exception e)
		{ }

		if (autoCheckbox.getState() == false)
		    showerSimulator.setOutFlowAndOutTemp();
		else
		    showerSimulator.simulateShower();
		repaint();
	}

	void coldPressureSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ coldPressureLabel3D.setText(String.valueOf(coldPressureSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void coldValveSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ 
		    if (autoCheckbox.getState() == false)
		    {   // manual mode ... set outFlow and outTemp
		        double coldValve = coldValveSlider.getValue()/100.0;
    		    showerSimulator.setColdValve(coldValve);
    		    showerSimulator.setOutFlowAndOutTemp();
    		    repaint();
    		}
		    else
		        // auto mode ... can't set the valve position manually
		        coldValveSlider.setValue((int)Math.round(showerSimulator.getColdValve()*100.0));
		    
		    coldValveLabel3D.setText(String.valueOf(coldValveSlider.getValue()));
		}
		catch (Exception e)
		{ }
		repaint();
	}

	void coldValveSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ coldValveLabel3D.setText(String.valueOf(coldValveSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void coldTempSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ coldTempLabel3D.setText(String.valueOf(coldTempSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void coldTempSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ 
    		// keep temp between 5 and 100
    		double coldTemp = (double)coldTempSlider.getValue();
    		if (coldTemp < 5.0)
    		{ coldTemp = 5.0;
    		  coldTempSlider.setValue((int)Math.round(coldTemp));
    		}
    		if (coldTemp > 65.0)
    		{ coldTemp = 65.0;
    		  coldTempSlider.setValue((int)Math.round(coldTemp));
    		}
    		showerSimulator.setColdTemp(coldTemp);
		    coldTempLabel3D.setText(String.valueOf(coldTempSlider.getValue()));
    	}
		catch (Exception e)
		{ }

		if (autoCheckbox.getState() == false)
		    showerSimulator.setOutFlowAndOutTemp();
		else
		    showerSimulator.simulateShower();
		repaint();
	}

	void showerTempSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ showerTempLabel3D.setText(String.valueOf(showerTempSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void showerTempSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ showerTempLabel3D.setText(String.valueOf(showerTempSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	void showerFlowSlider_propertyChange(java.beans.PropertyChangeEvent event)
	{
		// to do: code goes here.
		try
		{ showerFlowLabel3D.setText(String.valueOf(showerFlowSlider.getValue()));
		}
		catch (Exception e)
		{ }
	}

	void showerFlowSlider_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
		try
		{ showerFlowLabel3D.setText(String.valueOf(showerFlowSlider.getValue()));}
		catch (Exception e)
		{ }
	}

	class SymItem implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object == autoCheckbox)
				autoCheckbox_ItemStateChanged(event);
			else if (object == showFiringsCheckbox)
				showFiringsCheckbox_ItemStateChanged(event);
		}
	}

	void autoCheckbox_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		// to do: code goes here.
		boolean state = autoCheckbox.getState();
		
		try
		{
    		if (state)
    		{   // auto mode
                setNoticeLabel("Auto Mode:  Fuzzy Control in effect");
                showerSimulator.simulateShower();
    		}
    		else
    		{   // manual mode
                setNoticeLabel("Manual Mode: Set Temperatures and Pressures of Hot and Cold feeds and select auto for fuzzy control");
            }
        }
        catch (Exception e)
        {}
		repaint();		
	}

	void showFiringsCheckbox_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		// to do: code goes here.
		if (showFiringsCheckbox.getState())
		    inferenceCycleResults.setVisible(true);
		else
		    inferenceCycleResults.setVisible(false);
	}

}

